/* erwin -- html editor
 * Copyright 1999 David Vogler <david@rhlx01.rz.fht-esslingen.de>
 *                Adrian Reber <adrian@rhlx01.rz.fht-esslingen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "erwin.h"
#include "license.h"

#define HTML_RED 255
#define HTML_GREEN 0
#define HTML_BLUE 0

#define STD_RED 0
#define STD_GREEN 0
#define STD_BLUE 0

#define QUOT_RED 0
#define QUOT_GREEN 255
#define QUOT_BLUE 0

extern char file[1024];
extern gchar *FileName;
extern GtkStyle *style;
extern GdkFont *font;
GdkFont *GPL_font = NULL;
extern GtkWidget *main_window;
gint changed = 1;
gint from_beginning = 0;

void
insert_text (char *string, gint length)
{
  gint position = 0;
  struct document *document;
  document = (g_list_nth (documents, gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main))))->data;
  document->changed = TRUE;
  gtk_text_freeze (GTK_TEXT (text));
  position = gtk_editable_get_position (GTK_EDITABLE (text));
  gtk_editable_insert_text (GTK_EDITABLE (text), string, length, &position);
  gtk_text_thaw (GTK_TEXT (text));
}

void
insert_text_around (gpointer text, struct change change, char *string, gint length)
{
  gchar *buffer;
  gchar *another_buffer;
  gint i = 0;
  struct change help;
  struct document *document;
  document = (g_list_nth (documents, gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main))))->data;
  document->changed = TRUE;
  help = change;
  /*now filling another_buffer with the starting tag */
  another_buffer = g_malloc (length + 9);
  another_buffer[0] = '<';
  for (i = 1; i < length + 1; i++)
    {
      another_buffer[i] = string[i - 1];
    }
  another_buffer[length + 1] = '>';
  another_buffer[length + 2] = '\0';
  buffer = gtk_editable_get_chars (text, change.start, change.end);
  gtk_editable_delete_selection (text);
  gtk_editable_insert_text (text, another_buffer, length + 2, &change.start);
  gtk_editable_insert_text (text, buffer, strlen (buffer), &change.start);
  /*now filling another_buffer with the ending tag */
  another_buffer[0] = '<';
  another_buffer[1] = '/';
  for (i = 2; i < length + 2; i++)
    {
      another_buffer[i] = string[i - 2];
    }
  another_buffer[length + 2] = '>';
  another_buffer[length + 3] = '\0';
  gtk_editable_insert_text (text, another_buffer, length + 3, &change.start);
  GTK_EDITABLE (text)->selection_start_pos = help.start;
  GTK_EDITABLE (text)->selection_end_pos = help.end;
  GTK_EDITABLE (text)->has_selection = 1;
  move_cursor (length + 2, RIGHT);
  g_free (buffer);
  g_free (another_buffer);
}

void
copy_to_clipboard (GtkWidget * window)
{
  IS_TEXT_NULL
    gtk_editable_copy_clipboard (GTK_EDITABLE (text));
}

void
cut_to_clipboard (GtkWidget * window)
{
  IS_TEXT_NULL
    gtk_editable_cut_clipboard (GTK_EDITABLE (text));
}

void
paste_from_clipboard (GtkWidget * window)
{
  IS_TEXT_NULL
    gtk_editable_paste_clipboard (GTK_EDITABLE (text));
}


struct change
test_selection (gpointer text)
{
  struct change change;
  gint help = 0;

  if (!GTK_EDITABLE (text)->has_selection)
    {
#ifdef DEBUG
      g_print ("\nNow no selection is made.\n");
#endif
      help = gtk_editable_get_position (text);
      change.start = change.end = help;
    }
  else
    {
#ifdef DEBUG
      g_print ("\nNow a selection is made.\n");
#endif

      change.start = 0;
      change.end = 0;
      change.start = GTK_EDITABLE (text)->selection_start_pos;
      change.end = GTK_EDITABLE (text)->selection_end_pos;
      if (change.start > change.end)
	{
	  help = change.start;
	  change.start = change.end;
	  change.end = help;
	}
    }
  return change;
}

gint
move_cursor (gint length, ErwinCursor orientation)
{
  gint txt_len = 0;
  struct change change;

  change = test_selection (text);
#ifdef DEBUG
  g_print ("\nNow moving cursor : %d", length);
#endif
  if (orientation == LEFT)
    {

      if ((change.start - length) <= 0)
	{
	  gtk_editable_set_position (GTK_EDITABLE(text), 0);
	}
      else
	{
	  gtk_editable_set_position (GTK_EDITABLE(text), change.start - length);
	}
#ifdef DEBUG
      g_print ("\nLEFT\n");
      g_print ("\nPosition is %d", change.start);
#endif
    }
  if (orientation == RIGHT)
    {

      txt_len = gtk_text_get_length (GTK_TEXT(text));
      if ((change.start + length) >= txt_len)
	{
	  gtk_editable_set_position (GTK_EDITABLE(text), txt_len);
	}
      else
	{
	  gtk_editable_set_position (GTK_EDITABLE(text), change.start + length);
	}
#ifdef DEBUG
      g_print ("\nRIGHT\n");
      g_print ("\nPosition is %d", change.start);
#endif
    }
  else
    {
      return 42;
    }
  return 0;
}

void
file_ok_sel (GtkWidget * widget, GtkFileSelection * fs)
{
  GString *only_the_file;

  only_the_file = g_string_new ("<img src=\"");
  only_the_file = g_string_append (only_the_file, g_basename (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs))));
  only_the_file = g_string_append (only_the_file, "\">");

  insert_text (only_the_file->str, only_the_file->len);
  move_cursor (only_the_file->len, RIGHT);
  g_string_free (only_the_file, TRUE);
  gtk_widget_destroy (GTK_WIDGET (fs));
}
void
img_functions (GtkWidget * window)
{
  GtkWidget *filew;
  IS_TEXT_NULL
    filew = gtk_file_selection_new ("Image File selection");
  gtk_signal_connect (GTK_OBJECT (filew), "destroy", (GtkSignalFunc) destroy_message, &filew);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), "clicked", (GtkSignalFunc) file_ok_sel, filew);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  /*gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), "nothing"); */
  gtk_widget_show (filew);
}

GdkColor *
get_main_tooltips_bgcolor (GdkWindow * window, GdkColormap * colormap)
{
  static GdkColor color;
  static gboolean alloced = FALSE;
  if (!alloced)
    {
      color.red = 0;
      color.green = 0;
      color.blue = 0;
      color.pixel = 2000;
      gdk_color_alloc (colormap, &color);
      alloced = TRUE;
    }
  return &color;
}

void
get_value_1 (GtkWidget * widget, GtkWidget * spinner)
{
  GtkSpinButton *spin;
  spin = GTK_SPIN_BUTTON (spinner);
  table_values.rows = gtk_spin_button_get_value_as_int (spin);
#ifdef DEBUG
  g_print ("\nspin button: %d\n", gtk_spin_button_get_value_as_int (spin));
#endif
}

void
get_value_2 (GtkWidget * widget, GtkWidget * spinner)
{
  GtkSpinButton *spin;
  spin = GTK_SPIN_BUTTON (spinner);
  table_values.cols = gtk_spin_button_get_value_as_int (spin);
#ifdef DEBUG
  g_print ("\nspin button: %d\n", gtk_spin_button_get_value_as_int (spin));
#endif
}


void
quit_program ()
{
  GtkWidget *message;
  GtkWidget *button1, *button2;
  struct document *document;
  document = (g_list_nth (documents, gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main))))->data;
  if (document->changed)
    {
#ifdef DEBUG
      g_print ("\nText has been changed after save event");
#endif
      message = gtk_message_new ();
      gtk_message_set_label (GTK_MESSAGE (message), "File has not been saved...\nReally want to quit?");
      gtk_message_set_nr_of_buttons (GTK_MESSAGE (message), 2);
      button1 = gtk_message_add_button (GTK_MESSAGE (message), 0, "Quit!");
      gtk_signal_connect (GTK_OBJECT (button1), "clicked",
			  GTK_SIGNAL_FUNC (quit_ask_no),
			  message);
      button2 = gtk_message_add_button (GTK_MESSAGE (message), 1, "Cancel!");
      gtk_signal_connect (GTK_OBJECT (button2), "clicked",
			  GTK_SIGNAL_FUNC (close_message_dialog),
			  message);
      gtk_widget_show (message);
      return;
    }
  really_quit_program ();

}

void
really_quit_program ()

{
  if (style)
    gtk_style_unref (style);
  if (font)
    gdk_font_unref (font);
#ifdef DEBUG
  g_print ("\n quit event occured\n");
#endif
  gtk_main_quit ();
}

gint
delete_event (GtkWidget * widget, GdkEvent * event, gpointer data)
{
#ifdef DEBUG
  g_print ("delete event occured\n");
#endif
  return (TRUE);
}

void
destroy_message (GtkWidget * dialog)
{
  gtk_grab_remove (dialog);
  gtk_widget_destroy (dialog);
}

void
destroy_table_wizard (GtkWidget * dialog)
{
  gint i = 1, ii = 1;
  gchar *buffer;
  gtk_editable_delete_selection (GTK_EDITABLE (text));
  buffer = gtk_editable_get_chars (GTK_EDITABLE (text), gtk_editable_get_position (GTK_EDITABLE (text)), -1);
  gtk_editable_delete_text (GTK_EDITABLE (text), gtk_editable_get_position (GTK_EDITABLE (text)), -1);
  insert_text ("<table>\n\n", 9);
  move_cursor (9, RIGHT);


  for (i = 1; i <= table_values.rows; i++)
    {

      insert_text ("<tr>\n</tr>\n", 11);


      move_cursor (5, RIGHT);




      for (ii = 1; ii <= table_values.cols; ii++)
	{
	  insert_text ("\t<td></td>\n", 11);


	  if (ii != table_values.cols || ii == 1)
	    {
	      move_cursor (11, RIGHT);
	    }
	  else
	    {
	      move_cursor (4, RIGHT);
	    }
	  if (table_values.cols == 1 && table_values.rows != 1)
	    {
	      move_cursor (1, LEFT);
	      insert_text ("\n", 1);
	      if (i == table_values.rows)
		{
		  move_cursor (7, RIGHT);
		  insert_text ("\n", 1);
		  move_cursor (1, RIGHT);
		}

	    }


	  if (i != table_values.rows && ii == table_values.cols)
	    {
	      move_cursor (8, RIGHT);
	    }
	  if (ii == table_values.cols)
	    {
	      move_cursor (15, RIGHT);
	    }
	  if (table_values.rows != 1 && table_values.cols != 1 && ii == table_values.cols)
	    {

	      move_cursor (4, RIGHT);
	      if (i != table_values.rows)
		{
		  insert_text ("\n", 1);
		  move_cursor ( 1, RIGHT);
		}
	      else
		{
		  move_cursor ( 7, RIGHT);
		  insert_text ("\n", 1);
		  move_cursor ( 1, RIGHT);
		}
	    }

	}
    }
  insert_text ("</table>\n", 9);
  ii = gtk_text_get_length (GTK_TEXT (text));
  gtk_editable_insert_text (GTK_EDITABLE (text), buffer, strlen (buffer), &ii);
  g_free (buffer);
  gtk_grab_remove (dialog);
  gtk_widget_destroy (dialog);
}

void
check_key (GtkWidget * widget, GdkEventKey * key)
{
  static gboolean next_time_back_to_old_color = FALSE;
  static gboolean first_time = TRUE;
  struct document *document;
  document = (g_list_nth (documents, gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main))))->data;
  if (first_time)
    {
      gtk_window_set_focus (GTK_WINDOW (main_window), text);
      first_time = FALSE;
    }
  if ((key->keyval >= 0x20) && (key->keyval <= 0xFF))
    {
#ifdef DEBUG
      g_print ("\nI got a %x", key->keyval);
#endif
      if ((key->keyval == 0xE4))
	{
	  /*insert_text ("&auml;", 6);
	     key->length = 0; *//* length ist set to 0 so no char appears */
	}
      if ((key->keyval == 0x3e))
	{
#ifdef DEBUG
	  g_print ("\nNext time we go back to the old color...");
#endif
	  next_time_back_to_old_color = TRUE;
	}
    }
  else
#ifdef DEBUG
    g_print ("\nI got a %d", key->keyval);
#endif
  if (next_time_back_to_old_color && (GTK_TOGGLE_BUTTON (syntax)->active))
    {
      insert_text (">", 1);
      move_cursor (1, RIGHT);
      apply_syntax_highlighting ();
      next_time_back_to_old_color = FALSE;
      key->length = 0;		/* length ist set to 0 so no char appears */
    }
  if ((key->keyval == 0x71)|| (key->keyval == LCTRL));
  else
    {
      document->changed = TRUE;
      document->saved = FALSE;

    }

}
void
find_entry_changed (GtkWidget * widget)
{
  from_beginning = 0;

}

void
open_file_ok (GtkWidget * widget, GtkFileSelection * fs)
{
  GtkWidget *scrollbar;
  struct document *document;
  gchar erwin[1025];
  FILE *fd;
  GString *blubber;
  if (nr_of_documents == -1)
    gtk_widget_show (GTK_WIDGET (notebook_html));
  if (is_file_a_directory (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs))))
    {
      blubber = g_string_new (strcat (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)), "/\0"));
      gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), blubber->str);
      g_string_free (blubber, TRUE);
      return;
    }

  fd = fopen (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)), "r");
#ifdef DEBUG
  g_print ("\nFilename is set to : %s", gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
#endif
  strcpy (file, gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));

  if (fd == NULL)
    {
#ifdef DEBUG
      g_print ("\nCan't open file for reading\n");
#endif
      display_dialog ("Error", "Insufficient permissions\n to write that file");
      return;

    }
  document = g_malloc (sizeof (struct document));
  text = gtk_text_new (NULL, NULL);
  g_print ("\nnew 1 text : %p", text);
  gtk_widget_set_style (GTK_WIDGET (text), style);
  gtk_text_set_editable (GTK_TEXT (text), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (text), TRUE);
  document->notebook_label = gtk_label_new (g_basename (file));
  gtk_widget_show (document->notebook_label);
  document->notebook_child = gtk_hbox_new (FALSE, FALSE);
  gtk_box_pack_start (GTK_BOX (document->notebook_child), text, TRUE, TRUE, 0);
  scrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
  gtk_box_pack_start (GTK_BOX (document->notebook_child), scrollbar, FALSE, FALSE, 0);
  gtk_widget_show (text);
  gtk_widget_show (scrollbar);
  gtk_widget_show (document->notebook_child);
  document->text = text;
  g_print ("\nnew 2 text : %p", document->text);
  document->filename = g_string_new (file);
  document->nr_of_notebook_child = ++nr_of_documents;
  document->notebook = notebook_main;
  document->changed = FALSE;
  document->saved = TRUE;
  document->close_file = FALSE;
  documents = g_list_append (documents, document);
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook_main), document->notebook_child, document->notebook_label);
  gtk_notebook_set_page (GTK_NOTEBOOK (notebook_main), document->nr_of_notebook_child);
  gtk_text_freeze (GTK_TEXT (text));
  while (fgets (erwin, 1024, fd) != NULL)
    {
      gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL,
		       erwin, strlen (erwin));
    }
  gtk_editable_set_position (GTK_EDITABLE (text), 0);
  gtk_text_thaw (GTK_TEXT (text));
  fclose (fd);
  set_title_name ();
  gtk_grab_remove (GTK_WIDGET (fs));
  gtk_widget_destroy (GTK_WIDGET (fs));

}

void
save_file_ok (GtkWidget * widget, GtkFileSelection * fs)
{
  GString *blubber;
  if (does_file_exist (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs))))
    {
      if (is_file_a_directory (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs))))
	{
	  blubber = g_string_new (strcat (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)), "/\0"));
	  gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), blubber->str);
	  g_string_free (blubber, TRUE);
	  return;
	}
      if (is_file_readonly (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs))))
	{
	  display_dialog ("Error", "Insufficient permissions\n to write that file");
	  return;
	}
      strcpy (file, gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
      gtk_widget_destroy (GTK_WIDGET (fs));
      save_file ();

    }
  else
    {

      if (is_file_readonly (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs))))
	{
	  display_dialog ("Error", "Insufficient permissions\n to write that file");
	  return;
	}
      strcpy (file, gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
      gtk_widget_destroy (GTK_WIDGET (fs));
      save_file ();
    }
#ifdef DEBUG
  g_print ("\ngboolean does_file_exist(gchar *) : %d", does_file_exist (file));
  g_print ("\ngboolean is_file_readonly(gchar *) : %d", does_file_exist (file));
  g_print ("\ngboolean is_file_a_directory(gchar *) : %d", does_file_exist (file));
#endif

}

void
new_file (void)
{
  GtkWidget *scrollbar;
  struct document *document;
  gchar *title;
  if (nr_of_documents == -1)
    gtk_widget_show (GTK_WIDGET (notebook_html));
  document = g_malloc (sizeof (struct document));
  text = gtk_text_new (NULL, NULL);
  g_print ("\nnew 1 text : %p", text);
  gtk_widget_set_style (GTK_WIDGET (text), style);
  gtk_text_set_editable (GTK_TEXT (text), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (text), TRUE);
  document->notebook_label = gtk_label_new ("Untitled");
  gtk_widget_show (document->notebook_label);
  document->notebook_child = gtk_hbox_new (FALSE, FALSE);
  gtk_box_pack_start (GTK_BOX (document->notebook_child), text, TRUE, TRUE, 0);
  scrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
  gtk_box_pack_start (GTK_BOX (document->notebook_child), scrollbar, FALSE, FALSE, 0);
  gtk_widget_show (text);
  gtk_widget_show (scrollbar);
  gtk_widget_show (document->notebook_child);
  document->text = text;
  g_print ("\nnew 2 text : %p", document->text);
  document->filename = g_string_new ("Untitled");
  document->nr_of_notebook_child = ++nr_of_documents;
  document->notebook = notebook_main;
  document->changed = TRUE;
  document->saved = FALSE;
  document->close_file = FALSE;
  documents = g_list_append (documents, document);
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook_main), document->notebook_child, document->notebook_label);
  gtk_notebook_set_page (GTK_NOTEBOOK (notebook_main), document->nr_of_notebook_child);
  body_functions ();
  strcpy (file, FILENAME);
  title = g_malloc (strlen (NAME) + strlen ("Untitled") + 3);
  strcpy (title, NAME);
  strncat (title, " - ", 3);
  strncat (title, "Untitled", strlen ("Untitled"));
  gtk_window_set_title (GTK_WINDOW (main_window), title);
  g_free (title);
}

void
close_message_dialog (GtkWidget * widget, gpointer data)
{
  GtkWidget *dialog = (GtkWidget *) data;
  gtk_widget_destroy (dialog);
}

void
ungrab_message (GtkWidget * widget, gpointer data)
{
  gtk_grab_remove (widget);
}

void
display_dialog (gchar * title, gchar * message)
{
  GtkWidget *label;
  GtkWidget *button;
  GtkWidget *dialog;

  dialog = gtk_dialog_new ();
  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
		      GTK_SIGNAL_FUNC (ungrab_message),
		      NULL);

  gtk_window_set_title (GTK_WINDOW (dialog), title);
  gtk_container_border_width (GTK_CONTAINER (dialog), 0);

  button = gtk_button_new_with_label ("Make it so!");

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (close_message_dialog),
		      dialog);

  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      button, FALSE, FALSE, 0);
  gtk_widget_grab_default (button);
  gtk_widget_show (button);

  label = gtk_label_new (message);

  gtk_misc_set_padding (GTK_MISC (label), 10, 10);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      label, TRUE, TRUE, 0);

  gtk_widget_show (label);


  gtk_widget_show (dialog);

  gtk_grab_add (dialog);
}


void
find_function (GtkWidget * widget, gpointer data)
{


  gint n;
  gchar *the_text;

#ifdef DEBUG

  g_print ("\nCase sensitive? %d", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find_options.case_sensitive)));
  g_print ("\nFrom beginning? %d", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find_options.from_beginning)));

#endif

  the_text = gtk_editable_get_chars (
				      GTK_EDITABLE (text),
				      (gint) 0,
			      (gint) gtk_text_get_length (GTK_TEXT (text)));
  if (to_be_searched)
    {
      g_free (to_be_searched);
    }
  to_be_searched = gtk_editable_get_chars (
					    GTK_EDITABLE (data), 0, -1);
  if (!strlen (to_be_searched))
    return;

  n = GTK_EDITABLE (text)->selection_end_pos;

  /* --- Find the string --- */
  n = look_for_string (the_text, n);

  if (n >= 0)
    {
      gtk_text_set_point (GTK_TEXT (text), n);
      gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, " ", 1);
      gtk_text_backward_delete (GTK_TEXT (text), 1);

      gtk_editable_select_region (GTK_EDITABLE (text),
				  n, n + strlen (to_be_searched));

      widget = NULL;
    }
  else
    {

      display_dialog ("Find...", "Not found.  Reached the end of the file.");
    }
  g_free (the_text);

}
gint
look_for_string (gchar * the_text, gint n)
{
  gint text_length;
  gint length;
  gint pos;
  gchar *to_be_searched_2, *the_text_2;

  text_length = (gint) strlen (the_text);
  length = (gint) strlen (to_be_searched);
  to_be_searched_2 = g_malloc (length);
  the_text_2 = g_malloc (length + 1);
  strcpy (to_be_searched_2, to_be_searched);
#ifdef DEBUG
  g_print ("\nfrom_beginning : %d", from_beginning);
#endif
  if (!from_beginning && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find_options.from_beginning)))
    {
      n = 0;
      from_beginning = 1;
    }
  for (pos = n; pos < text_length - length; pos++)
    {
      strncpy (the_text_2, &the_text[pos], length);
      the_text_2[length] = '\0';

      if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find_options.case_sensitive)))
	{
	  g_strdown (the_text_2);
	  g_strdown (to_be_searched_2);
#ifdef DEBUG
	  g_print ("\nthe_text_2 : %s", the_text_2);
#endif
	  if (!strcmp (the_text_2, to_be_searched_2))
	    {
	      g_free (the_text_2);
	      g_free (to_be_searched_2);
	      return (pos);
	    }
	}
      else
	{

#ifdef DEBUG
	  g_print ("\nthe_text_2 : %s", the_text_2);
#endif

	  if (!strcmp (the_text_2, to_be_searched_2))
	    {
	      g_free (the_text_2);
	      g_free (to_be_searched_2);
	      return (pos);
	    }
	}

    }
  g_free (the_text_2);
  g_free (to_be_searched_2);
  return (-1);
}

GtkWidget *
entry_in_frame (gchar * title, struct frame_entry * frame_entry)
{
  GtkWidget *entry_box;
  frame_entry->frame = gtk_frame_new (title);
  entry_box = gtk_hbox_new (TRUE, TRUE);
  gtk_container_add (GTK_CONTAINER (frame_entry->frame), entry_box);
  frame_entry->entry = gtk_entry_new ();
  gtk_widget_show (frame_entry->entry);
  gtk_box_pack_start (GTK_BOX (entry_box), frame_entry->entry, FALSE, FALSE, 30);
  gtk_widget_show (entry_box);
  gtk_widget_show (frame_entry->frame);
  return entry_box;
}

void
body_functions ()
{
  body_dialog (body_elements);
}

void
legal ()

{

  GtkWidget *license_text;
  GtkWidget *button;
  GtkWidget *dialog;
  GtkWidget *hbox;
  GtkWidget *scroll;

  dialog = gtk_dialog_new ();
  gtk_widget_set_usize (GTK_WIDGET (dialog),
			(gint) (0.6 * gdk_screen_width ()),
			(gint) (0.3 * gdk_screen_height ()));
  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
		      GTK_SIGNAL_FUNC (ungrab_message),
		      NULL);

  gtk_window_set_title (GTK_WINDOW (dialog), "Legal/License/About");
  gtk_container_border_width (GTK_CONTAINER (dialog), 0);

  button = gtk_button_new_with_label ("Make it so!");

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (close_message_dialog),
		      dialog);

  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      button, FALSE, FALSE, 0);
  gtk_widget_grab_default (button);
  gtk_widget_show (button);
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, TRUE, TRUE, 0);
  license_text = gtk_text_new (NULL, NULL);
  scroll = gtk_vscrollbar_new (GTK_TEXT (license_text)->vadj);

  gtk_box_pack_start (GTK_BOX (hbox),
		      license_text, TRUE, TRUE, 0);
  gtk_text_set_editable (GTK_TEXT (license_text), FALSE);
  if (!GPL_font)
    {
      GPL_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-120-*-*-*-*-*-*");
    }
  gtk_text_set_word_wrap (GTK_TEXT (license_text), FALSE);

  gtk_text_insert (GTK_TEXT (license_text), GPL_font, NULL, NULL, license, strlen (license));
  gtk_widget_show (license_text);
  gtk_box_pack_start (GTK_BOX (hbox), scroll, FALSE, TRUE, 0);
  gtk_widget_show (scroll);
  gtk_widget_show (hbox);
  gtk_widget_show (dialog);

  gtk_grab_add (dialog);

}

void
set_title_name ()
{
  GString *title;
#ifdef DEBUG
  g_print ("\nfilename : %s", file);
  g_print ("\nbasename : %s", g_basename (file));
#endif
  title = g_string_new (NAME);
  title = g_string_append (title, " - ");
  if (strcmp (file, FILENAME))
    title = g_string_append (title, g_basename (file));
  else
    title = g_string_append (title, "Untitled");
  gtk_window_set_title (GTK_WINDOW (main_window), title->str);
  g_string_free (title, TRUE);
}


void
ok_body_dialog (GtkWidget * widget, gpointer data)
{
  gchar *title1 = "<html>\n<head>";
  gchar *title2 = "\n<title>";
  gchar *title3 = "</title>\n</head>\n\n<body";
  gchar *title4 = ">\n\n</body>\n</html>";
  GString *title;
  GString *body_options;

  gtk_editable_delete_text (GTK_EDITABLE (text), 0, -1);
  gtk_text_freeze (GTK_TEXT (text));

  body_options = g_string_new ("");

  /* Now testing background color */

  make_body_options_string (body_options, " bgcolor=\"", &body_elements.bgcolor);

/*-------------------------------------------------------------------------*/
  /* Now testing text color */

  make_body_options_string (body_options, " text=\"", &body_elements.text);

/*-------------------------------------------------------------------------*/

  /* Now testing link color */

  make_body_options_string (body_options, " link=\"", &body_elements.link);

/*-------------------------------------------------------------------------*/
  /* Now testing vlink color */

  make_body_options_string (body_options, " vlink=\"", &body_elements.vlink);

/*-------------------------------------------------------------------------*/
  /* Now testing alink color */

  make_body_options_string (body_options, " alink=\"", &body_elements.alink);

/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
  /* Now testing backgroundimage */

  make_body_options_string (body_options, " background=\"", &body_elements.background);

/*-------------------------------------------------------------------------*/


#ifdef DEBUG
  g_print ("\nbody_elements.title.entry : %s",
	   (gtk_entry_get_text (GTK_ENTRY (body_elements.title.entry))));
  g_print ("\nbody_elements.bgcolor.entry : %s",
	   (gtk_entry_get_text (GTK_ENTRY (body_elements.bgcolor.entry))));
  g_print ("\nbody_elements.text.entry : %s",
	   (gtk_entry_get_text (GTK_ENTRY (body_elements.text.entry))));
#endif
  /* Making the string that will be inserted */
  title = g_string_new (title1);
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (body_elements.insert_generator)))
    title = g_string_append (title, "\n  <meta name=\"GENERATOR\" content=\"" NAME "\">");
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (body_elements.content_type)))
    title = g_string_append (title, "\n  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">");
  title = g_string_append (title, title2);
  title = g_string_append (title, gtk_entry_get_text
			   (GTK_ENTRY (body_elements.title.entry)));
  title = g_string_append (title, title3);
  title = g_string_append (title, body_options->str);
  title = g_string_append (title, title4);
  /* String made... Now inserting */
  insert_text (title->str, title->len);
  gtk_text_thaw (GTK_TEXT (text));
  /*Freeing memory */
  g_string_free (title, TRUE);
  g_string_free (body_options, TRUE);
  close_body_dialog (NULL, GTK_WIDGET (data));
}

void
make_body_options_string (GString * body_options, gchar * text, struct frame_entry *element)
{

  if (strcmp ("", (gtk_entry_get_text (GTK_ENTRY (element->entry)))))
    {
      body_options = g_string_append (body_options, text);
      body_options = g_string_append (body_options, gtk_entry_get_text (GTK_ENTRY (element->entry)));
      body_options = g_string_append (body_options, "\"");
    }
}

gboolean
does_file_exist (gchar * filename)
{
  struct stat buf;
  if (stat (filename, &buf) == -1)
    return FALSE;
  return TRUE;
}

gboolean
is_file_readonly (gchar * filename)
{
  if (strcmp (filename, FILENAME) == 0 || access (filename, F_OK))
    return FALSE;

  return (access (filename, W_OK) != 0);
}

gboolean
is_file_a_directory (gchar * filename)
{
  struct stat buf;
  stat (filename, &buf);
  if (buf.st_mode & S_IFREG)
    return FALSE;
  return TRUE;

}

void
insert_symmetric_tag (gchar * tag)
{
  struct change change;
  GString *together;
  struct document *document;
  document = (g_list_nth (documents, gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main))))->data;
  document->changed = TRUE;
  change = test_selection (text);
  gtk_text_freeze (GTK_TEXT (text));

  if (!(change.start - change.end))
    {
      together = g_string_new ("<");
      together = g_string_append (together, tag);
      together = g_string_append (together, "></");
      together = g_string_append (together, tag);
      together = g_string_append (together, ">");
      insert_text (together->str, together->len);
      g_string_free (together, TRUE);
      move_cursor (strlen (tag) + 2, RIGHT);
    }
  else
    {
      insert_text_around (text, change, tag, strlen (tag));
    }
  gtk_text_thaw (GTK_TEXT (text));
  gtk_window_set_focus (GTK_WINDOW (main_window), text);
}

void
body_img_function (GtkWidget * window)
{
  GtkWidget *filew;
  gtk_grab_remove (body_elements.dialog);
  filew = gtk_file_selection_new ("Image File selection");
  gtk_signal_connect (GTK_OBJECT (filew), "destroy", (GtkSignalFunc) destroy_message, &filew);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), "clicked", (GtkSignalFunc) body_img_file_ok_sel, filew);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_widget_show (filew);
}

void
body_img_file_ok_sel (GtkWidget * widget, GtkWidget * fs)
{

  gtk_entry_set_text (GTK_ENTRY (body_elements.background.entry), g_basename (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs))));
  gtk_widget_destroy (GTK_WIDGET (fs));
  gtk_grab_add (body_elements.dialog);
}

void
href_function_ok (GtkWidget * widget, gpointer data)
{
  GString *new_string;

  gtk_text_freeze (GTK_TEXT (text));
  gtk_editable_delete_selection (GTK_EDITABLE (text));
  new_string = g_string_new ("<a href=\"");	/* You can see the making */
  if (strcmp ("none", gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (hyperlink.protocol.combo)->entry))))
    new_string = g_string_append (new_string, gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (hyperlink.protocol.combo)->entry)));
  new_string = g_string_append (new_string, gtk_entry_get_text (GTK_ENTRY (hyperlink.link.entry)));
  new_string = g_string_append (new_string, "\">");
  new_string = g_string_append (new_string, gtk_entry_get_text (GTK_ENTRY (hyperlink.name.entry)));
  new_string = g_string_append (new_string, "</a>");	/* new string */
  gtk_text_thaw (GTK_TEXT (text));
  insert_text (new_string->str, new_string->len);
  move_cursor (new_string->len - 4, RIGHT);
  g_string_free (new_string, TRUE);
  close_body_dialog (NULL, GTK_WIDGET (data));
  gtk_window_set_focus (GTK_WINDOW (main_window), text);
}

GtkWidget *
combo_in_frame (gchar * title, struct frame_combo *frame_combo, GList * list_items)
{
  GtkWidget *combo_box;
  frame_combo->frame = gtk_frame_new (title);
  combo_box = gtk_hbox_new (TRUE, TRUE);
  gtk_container_add (GTK_CONTAINER (frame_combo->frame), combo_box);
  frame_combo->combo = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (frame_combo->combo), list_items);
  frame_combo->entry = (GTK_COMBO (frame_combo->combo)->entry);
  gtk_entry_set_editable (GTK_ENTRY (frame_combo->entry), FALSE);
  gtk_widget_show (frame_combo->combo);
  gtk_box_pack_start (GTK_BOX (combo_box), frame_combo->combo, FALSE, FALSE, 30);
  gtk_widget_show (combo_box);
  gtk_widget_show (frame_combo->frame);
  return combo_box;
}

void
font_function_ok (GtkWidget * widget, gpointer data)
{
  struct change change;
  struct change help;
  gchar *title1 = "<font";
  gchar *title2 = "</font>";
  GString *font_;
  GString *font_option;
  gchar *selected;

  if (GTK_EDITABLE (text)->has_selection)
    {
      change = test_selection (text);	/* Getting start and end position */
      help = change;
      selected = gtk_editable_get_chars (GTK_EDITABLE (text), change.start, change.end);
      gtk_editable_delete_selection (GTK_EDITABLE (text));
    }
  else
    selected = NULL;

  gtk_text_freeze (GTK_TEXT (text));

  font_option = g_string_new ("");
  /* Now testing font color */

  make_body_options_string (font_option, " color=\"", &font_options.color);

/*-------------------------------------------------------------------------*/
  /* Now testing font face */

  make_body_options_string (font_option, " face=\"", &font_options.face);

/*-------------------------------------------------------------------------*/
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (font_options.size)))
    {
      font_option = g_string_append (font_option, " size=\"");
      font_option = g_string_append_c (font_option, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (font_options.size)) + 48);
      font_option = g_string_append (font_option, "\"");
    }
  font_ = g_string_new (title1);
  font_ = g_string_append (font_, font_option->str);
  font_ = g_string_append (font_, ">");
  if (selected)
    font_ = g_string_append (font_, selected);
  font_ = g_string_append (font_, title2);
  /* String made... Now inserting */
  insert_text (font_->str, font_->len);
  move_cursor (font_->len - 7, RIGHT);
  if (selected)
    {
      g_free (selected);
    }

  gtk_text_thaw (GTK_TEXT (text));
  /*Freeing memory */
  g_string_free (font_, TRUE);
  g_string_free (font_option, TRUE);
  close_body_dialog (NULL, GTK_WIDGET (data));
  gtk_window_set_focus (GTK_WINDOW (main_window), text);
}

gushort
convert_color (guint c)
{
  if (c == 0)
    return (0);
  c *= 257;
  return (c > 0xffff) ? 0xffff : c;
}

void
extract_color (GdkColor * color, guint red, guint green, guint blue)
{
  color->red = convert_color (red);
  color->green = convert_color (green);
  color->blue = convert_color (blue);
}

/* void apply_syntax_highlighting
 * - the syntax highlighting is very primitive
 *   the whole text is deleted and then I am inserting it again
 *   while looking for "<" and ">". If "<" and ">" is found the
 *   text is inserted with a new color...
 * - before the highlighting is actual executed I check is every
 *   "<" has a corresponding ">", if not ...  nothing happens.
 */

void
apply_syntax_highlighting ()
{
  GdkColormap *cmap;
  GdkColor html;
  gint i;
  gint begin = -1;
  gint end = -1;
  gint begin_2 = -2;
  gint end_2 = -2;
  gint counter1 = 0;
  gint counter2 = 0;
  gchar one_char;
  gboolean end_found = FALSE;
  gboolean second = FALSE;
  gboolean first = FALSE;
  gboolean mark_set = FALSE;
  gboolean inside_of_tag = FALSE;
  gint mark = 0;
  GString *colored_text;
  GString *the_text;
  gint point;
  IS_TEXT_NULL
#ifdef DEBUG
    g_print ("\nNow highlighting syntax");
#endif
  gtk_text_freeze (GTK_TEXT (text));
  point = gtk_editable_get_position (GTK_EDITABLE (text));
  the_text = g_string_new (gtk_editable_get_chars (GTK_EDITABLE (text), 0, -1));
#ifdef DEBUG
  g_print ("\npoint : %d\nthe_text->len : %d", point, the_text->len);
#endif

  for (i = 0; i < the_text->len; i++)
    {
      one_char = the_text->str[i];
      if (one_char == 60)
	{
	  counter1++;
	  if (first && !second)
	    {
	      g_string_free (the_text, TRUE);
	      gtk_text_thaw (GTK_TEXT (text));
	      return;
	    }

	  first = TRUE;
	  second = FALSE;
	}
      if (one_char == 62)
	{
	  if (!first && second)
	    {
	      g_string_free (the_text, TRUE);
	      gtk_text_thaw (GTK_TEXT (text));
	      return;
	    }
	  second = TRUE;
	  first = FALSE;
	}
    }
  if (!counter1)
    return;
  else
    counter1 = 0;
  gtk_editable_delete_text (GTK_EDITABLE (text), 0, -1);
  for (i = 0; i < the_text->len; i++)
    {
      one_char = the_text->str[i];
      if (one_char != 60 && !i)
	{
	  begin_2 = -1;
	}
      if (one_char == 60)	/* searching for < */
	{
	  counter1++;
	  begin = i;
	  end_2 = i;
	  inside_of_tag = TRUE;
	  mark_set = FALSE;
#ifdef DEBUG
	  g_print ("\ntag begin is : %d", begin);
#endif
	}
      if (one_char == 62)	/*searching for > */
	{
	  counter2++;
	  end = i;
	  begin_2 = i;
	  inside_of_tag = FALSE;
#ifdef DEBUG
	  g_print ("\ntag end is : %d", end);
#endif
	  end_found = TRUE;
	  mark_set = FALSE;
	}
      if (i == the_text->len - 1)
	{

	  end_2 = the_text->len - 1;
	  end_found = TRUE;

	}
      if (one_char == 34 && inside_of_tag)
	{
	  if (!mark_set)
	    {
	      mark_set = TRUE;
	      mark = i;
	      end = i - 1;
	      end_found = TRUE;

	    }
	  else
	    {
	      begin = i + 1;
	      /* Making string which is getting inserted */
	      colored_text = g_string_new ((the_text->str) + mark);
	      colored_text = g_string_truncate (colored_text, begin - mark);
	      /* Color */
	      cmap = gdk_colormap_get_system ();
	      extract_color (&html, QUOT_RED, QUOT_GREEN, QUOT_BLUE);
	      gdk_color_alloc (cmap, &html);
	      /* Inserting string with wanted color */
	      gtk_text_insert (GTK_TEXT (text), NULL, &html, NULL, colored_text->str, colored_text->len);
	      g_string_free (colored_text, TRUE);
	      mark_set = FALSE;

	    }
	}
      if (end_found && (begin + 1) && (end + 1) && (begin <= end))
	/* this must be a html tag */
	{
	  end_found = FALSE;
#ifdef DEBUG
	  g_print ("\nSome figures:\nbegin : %d....end : %d", begin, end);
#endif

	  /* Making string which is getting inserted */
	  colored_text = g_string_new ((the_text->str) + begin);
	  colored_text = g_string_truncate (colored_text, end - begin + 1);
	  /* Color */
	  cmap = gdk_colormap_get_system ();
	  extract_color (&html, HTML_RED, HTML_GREEN, HTML_BLUE);
	  gdk_color_alloc (cmap, &html);
	  /* Inserting string with wanted color */
	  gtk_text_insert (GTK_TEXT (text), NULL, &html, NULL, colored_text->str, colored_text->len);
	  g_string_free (colored_text, TRUE);
	  begin = -1;
	  end = -1;
	}
      if ((begin_2 < end_2) && (begin_2 + 2) && (end_2 + 2))
	/* this is text outside of a tag */
	{
#ifdef DEBUG
	  g_print ("\nSome figures:\nbegin_2 : %d....end_2 : %d", begin_2, end_2);
#endif
	  /* Making string which is getting inserted */
	  colored_text = g_string_new ((the_text->str) + begin_2 + 1);
	  colored_text = g_string_truncate (colored_text, end_2 - begin_2 - 1);
	  /* Color */
	  cmap = gdk_colormap_get_system ();
	  extract_color (&html, STD_RED, STD_GREEN, STD_BLUE);
	  gdk_color_alloc (cmap, &html);
	  /* Inserting string with wanted color */
	  gtk_text_insert (GTK_TEXT (text), NULL, &html, NULL, colored_text->str, colored_text->len);
	  g_string_free (colored_text, TRUE);
	  begin_2 = -2;
	  end_2 = -2;
	}
    }
#ifdef DEBUG
  g_print ("\n%d < found", counter1);
  g_print ("\n%d > found", counter2);
#endif
  g_string_free (the_text, TRUE);
  cmap = gdk_colormap_get_system ();
  extract_color (&html, STD_RED, STD_GREEN, STD_BLUE);
  gdk_color_alloc (cmap, &html);
#ifdef DEBUG
  g_print ("\npoint : %d\nthe_text->len : %d", point, the_text->len);
#endif
  gtk_editable_set_position (GTK_EDITABLE (text), point);
  gtk_text_thaw (GTK_TEXT (text));

}

/* void undo_syntax_highlighting
 * - the whole text is deleted and inserted again with the standard 
 *   color (maybe not the best way)
 * - if I freeze and thaw the text I always get segfaults ...
 */

void
undo_syntax_highlighting ()
{
  GString *the_text;
  gint point;
  IS_TEXT_NULL
  /* gtk_text_freeze(GTK_TEXT(text)); */
    point = gtk_editable_get_position (GTK_EDITABLE (text));
  the_text = g_string_new (gtk_editable_get_chars (GTK_EDITABLE (text), 0, -1));
  gtk_editable_delete_text (GTK_EDITABLE (text), 0, -1);
  gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, the_text->str, the_text->len);
  g_string_free (the_text, TRUE);
  gtk_editable_set_position (GTK_EDITABLE (text), point);
  /* gtk_text_thaw(GTK_TEXT(text)); */
}

/* void switch_page
 * - this is the callback when the event switch_page occurs
 * - notebook_label, text, title are set according to the
 *   values retrieved from the GList ...
 * - new text is focused
 */

void
switch_page (GtkWidget * widget, GtkNotebookPage * page, gint page_num)
{
  struct document *document;
  if (!nr_of_documents)
    page_num = 0;
#ifdef DEBUG
  g_print ("\nLength of list : %d", g_list_length (documents));
  g_print ("\npage_num : %d", page_num);
#endif
  document = (g_list_nth (documents, page_num))->data;
  text = document->text;
  strcpy (file, document->filename->str);
  set_title_name ();
  gtk_window_set_focus (GTK_WINDOW (main_window), text);
}

/*  void close_file 
 *   - checking if current document has changed
 *     if yes, then ask user if would like to save file
 */

void
close_file ()
{
  GtkWidget *message;
  GtkWidget *button1, *button2, *button3;
  struct document *document;
  if (nr_of_documents == -1)
    return;
  document = (g_list_nth (documents, gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main))))->data;
  if (document->changed)
    {
#ifdef DEBUG
      g_print ("\nText has been changed after save event");
#endif
      message = gtk_message_new ();
      gtk_message_set_label (GTK_MESSAGE (message), "File has changed since last save event!\nDo you want to save the file now?");
      gtk_message_set_nr_of_buttons (GTK_MESSAGE (message), 3);
      button1 = gtk_message_add_button (GTK_MESSAGE (message), 0, "Yes");
      gtk_signal_connect (GTK_OBJECT (button1), "clicked",
			  GTK_SIGNAL_FUNC (close_ask_yes),
			  message);
      button2 = gtk_message_add_button (GTK_MESSAGE (message), 1, "No");
      gtk_signal_connect (GTK_OBJECT (button2), "clicked",
			  GTK_SIGNAL_FUNC (close_ask_no),
			  message);
      button3 = gtk_message_add_button (GTK_MESSAGE (message), 2, "Hmm?");
      gtk_signal_connect (GTK_OBJECT (button3), "clicked",
			  GTK_SIGNAL_FUNC (close_message_dialog),
			  message);
      gtk_widget_show (message);
      return;
    }
  really_close_file ();

}

/* void switch_html_page
 * - sets the focus on the text widget
 */

void
switch_html_page ()
{
  IS_TEXT_NULL
    gtk_window_set_focus (GTK_WINDOW (main_window), text);
}

/*  void really_close_file
 *  - current page of the notebook is removed
 *    and gint nr_of_notebook_child in the GList *documents
 *    gets new values...
 *  - the removed struct document *document is freed
 *  - the new value for gint nr_of_documents is assigned
 *  - GtkWidget *text is NULL if no more documents are opened
 */

void
really_close_file ()
{
  gint page_num = 0;
  struct document *document;
  document = (g_list_nth (documents, gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main))))->data;
  if (!nr_of_documents && document->saved)
    {
      gtk_editable_delete_text (GTK_EDITABLE (text), 0, -1);
      document->filename = g_string_new ("Untitled");
      strcpy (file, FILENAME);
      set_title_name ();
      gtk_label_set_text (GTK_LABEL (document->notebook_label), "Untitled");
    }
  if (!nr_of_documents)
    return;
  documents = g_list_remove (documents, document);
  g_free (document);
  nr_of_documents--;
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook_main), gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main)));
  for (page_num = 0; page_num <= nr_of_documents; page_num++)
    {
      document = (g_list_nth (documents, page_num))->data;
      document->nr_of_notebook_child = page_num;
    }
  if (nr_of_documents == -1)
    {
      text = NULL;
      gtk_widget_hide (GTK_WIDGET (notebook_html));
    }
}

void
close_ask_no (GtkWidget * widget, gpointer data)
{
  GtkWidget *dialog = (GtkWidget *) data;
  struct document *document;
  document = (g_list_nth (documents, gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main))))->data;
  document->saved = TRUE;
  gtk_widget_destroy (dialog);
  really_close_file ();
}

void
close_ask_yes (GtkWidget * widget, gpointer data)
{
  GtkWidget *dialog = (GtkWidget *) data;
  struct document *document;
  document = (g_list_nth (documents, gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main))))->data;
  document->close_file = TRUE;
  gtk_widget_destroy (dialog);
  save_file ();
  really_close_file ();
}

void
quit_ask_no (GtkWidget * widget, gpointer data)
{
  GtkWidget *dialog = (GtkWidget *) data;
  struct document *document;
  document = (g_list_nth (documents, gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook_main))))->data;
  document->saved = TRUE;
  gtk_widget_destroy (dialog);
  really_quit_program ();
}

void
quit_ask_yes (GtkWidget * widget, gpointer data)
{
  GtkWidget *dialog = (GtkWidget *) data;
  QUIT = TRUE;
  gtk_widget_destroy (dialog);
  save_file ();
  really_quit_program ();
}
