/* erwin -- html editor
 * Copyright 1999 David Vogler <david@rhlx01.rz.fht-esslingen.de>
 *                Adrian Reber <adrian@rhlx01.rz.fht-esslingen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


#include "erwin.h"
#include "erwinpixmaps.h"

GtkWidget *
main_toolbar (GtkWidget * main_window)
{

  GtkWidget *toolbar, *pixmap_wid;
  GtkStyle *style;
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  GdkWindow *window;

  toolbar = gtk_toolbar_new (GTK_ORIENTATION_VERTICAL, GTK_TOOLBAR_ICONS);
  style = gtk_widget_get_style (main_window);

  window = main_window->window;

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_new_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "New", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (new_file),
			    NULL);

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_open_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Open", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (open_file_dlg),
			    NULL);


  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_save_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Save", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (save_file),
			    NULL);
  /* Close Button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_cancel_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "close", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (close_file),
			    NULL);
  /* Quit Button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_exit_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Quit", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (quit_program),
			    NULL);


  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_cut_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Cut\tCtrl+X", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (cut_to_clipboard),
			    NULL);


  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_copy_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Copy\tCtrl+C", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (copy_to_clipboard),
			    NULL);


  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_paste_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Paste\tCtrl+V", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (paste_from_clipboard),
			    NULL);
  /* Search Button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) search_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Search", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (find_dlg),
			    NULL);
  /* SPACE */
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  /* Syntax Highlighting */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) light1_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  syntax = gtk_toolbar_append_element (
					GTK_TOOLBAR (toolbar),
					GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, NULL, "Enable Syntax Highlighting", NULL,
					pixmap_wid,
					NULL,
					NULL);


  return toolbar;
}

GtkWidget *
html_toolbar (GtkWidget * main_window, GtkWidget * text)
{
  GtkWidget *toolbar, *pixmap_wid;
  GtkStyle *style;
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  GdkWindow *window;

  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  style = gtk_widget_get_style (main_window);

  window = main_window->window;

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) quick_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Quickstart", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (body_functions),
			    NULL);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_linebreak);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<BR>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (br_functions),
			    NULL);
/* PARAGRAPH */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) paragraph);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<P>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (p_functions),
			    NULL);
/*NOBREAKING SPACE */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) space);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "&nbsp;", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (nbsp_functions),
			    NULL);



/* SPACE */
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);


  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_link);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<A HREF=\"\"></A>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (href_dialog),
			    NULL);

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_image);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "insert image link", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (img_functions),
			    NULL);
/* SPACE */
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  /* EMPHASIS */

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_emphasis);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<EM>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (em_functions),
			    NULL);
  /*STRONG */

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_strong);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<STRONG>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (strong_functions),
			    NULL);
/*SMALL */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_small);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<SMALL>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (small_functions),
			    NULL);
/*STRIKE */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_strikeout);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<STRIKE>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (strike_functions),
			    NULL);
/*SUB */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_subscript);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<SUB>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (sub_functions),
			    NULL);
/*SUP */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_superscript);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<SUP>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (sup_functions),
			    NULL);



  //gtk_window_set_focus(GTK_WINDOW(main_window),text);
  return toolbar;

}
GtkWidget *
font_toolbar (GtkWidget * main_window, GtkWidget * text)
{
  GtkWidget *toolbar, *pixmap_wid;
  GtkStyle *style;
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  GdkWindow *window;

  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  style = gtk_widget_get_style (main_window);

  window = main_window->window;



/*---menus for h1...h3---*/

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_h1);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<H1>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (h1_functions),
			    NULL);

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_h2);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<H2>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (h2_functions),
			    NULL);

  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_h3);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<H3>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (h3_functions),
			    NULL);
  /*Space in the toolbar */
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  /*center button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) center);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<CENTER>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (center_functions),
			    NULL);
  /*Space in the toolbar */
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  /*bold button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_bold);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<B>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (bold_functions),
			    NULL);
  /*italic button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_italic);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<I>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (italic_functions),
			    NULL);
  /*underline button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tb_underline);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<U>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (underline_functions),
			    NULL);
  /*Space in the toolbar */
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  /*fontface button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) font);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Font Face", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (font_dialog),
			    NULL);

  //gtk_window_set_focus(GTK_WINDOW(main_window),text);
  return toolbar;

}

GtkWidget *
table_toolbar (GtkWidget * main_window, GtkWidget * text)
{
  GtkWidget *toolbar, *pixmap_wid;
  GtkStyle *style;
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  GdkWindow *window;

  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  style = gtk_widget_get_style (main_window);

  window = main_window->window;



  /*table wizard button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) html_table_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "TABLE-WIZARD", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (table_wizard),
			    NULL);
  /*Space in the toolbar */
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  /*Table button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tbl_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<TABLE>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (table_functions),
			    NULL);
/*th button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) th_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<TH>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (th_functions),
			    NULL);

/*TR button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) tr_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<TR>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (tr_functions),
			    NULL);
  /*td button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) td_xpm);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "<TD>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (td_functions),
			    NULL);

  /*Space in the toolbar */
  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  /*OL button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) orderedlist);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Ordered List <OL>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (ol_functions),
			    NULL);
/*UL button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) unorderedlist);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Unordered List <UL>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (ul_functions),
			    NULL);

/*LI button */
  gtk_tooltips_set_colors (GTK_TOOLBAR (toolbar)->tooltips,
			   get_main_tooltips_bgcolor
		    (main_window->window, gdk_window_get_colormap (window)),
			   &style->fg[GTK_STATE_NORMAL]);

  gtk_tooltips_set_delay (GTK_TOOLBAR (toolbar)->tooltips, 100);

  pixmap = gdk_pixmap_create_from_xpm_d (
					  main_window->window,
					&mask, &style->bg[GTK_STATE_NORMAL],
					  (gchar **) listitem);

  pixmap_wid = gtk_pixmap_new (pixmap, mask);
  gtk_widget_show (pixmap_wid);

  gtk_toolbar_append_item (
			    GTK_TOOLBAR (toolbar),
			    NULL, "Listitem <LI>", NULL,
			    pixmap_wid,
			    GTK_SIGNAL_FUNC (li_functions),
			    NULL);



  //gtk_window_set_focus(GTK_WINDOW(main_window),text);
  return toolbar;

}
