/* GTKMESSAGE

 * Adrian Reber <adrian@rhlx01.fht-esslingen.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>
#include "gtkmessage.h"


static void gtk_message_class_init (GtkMessageClass * klass);
static void gtk_message_init (GtkMessage * message);


GtkType
gtk_message_get_type (void)
{
  static GtkType message_type = 0;

  if (!message_type)
    {
      static const GtkTypeInfo message_info =
      {
	"GtkMessage",
	sizeof (GtkMessage),
	sizeof (GtkMessageClass),
	(GtkClassInitFunc) gtk_message_class_init,
	(GtkObjectInitFunc) gtk_message_init,
      /* reserved_1 */ NULL,
      /* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      message_type = gtk_type_unique (GTK_TYPE_WINDOW, &message_info);
    }

  return message_type;
}

static void
gtk_message_class_init (GtkMessageClass * class)
{
}

static void
gtk_message_init (GtkMessage * message)
{
  GtkWidget *separator;

  message->vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (message), message->vbox);
  gtk_widget_show (message->vbox);
  message->label = gtk_label_new ("");
  gtk_container_add (GTK_CONTAINER (message->vbox), message->label);
  gtk_widget_show (message->label);
  message->action_area = gtk_hbox_new (TRUE, 5);
  gtk_container_set_border_width (GTK_CONTAINER (message->action_area), 10);
  gtk_box_pack_end (GTK_BOX (message->vbox), message->action_area, FALSE, TRUE, 0);
  gtk_widget_show (message->action_area);

  separator = gtk_hseparator_new ();
  gtk_box_pack_end (GTK_BOX (message->vbox), separator, FALSE, TRUE, 0);
  gtk_widget_show (separator);
}

GtkWidget *
gtk_message_new (void)
{
  return GTK_WIDGET (gtk_type_new (GTK_TYPE_MESSAGE));
}

void
gtk_message_set_label (GtkMessage * message, const gchar * label)
{
  gtk_label_set_text (GTK_LABEL (message->label), label);
}

void
gtk_message_set_nr_of_buttons (GtkMessage * message, gint size)
{
  g_return_if_fail (message != NULL);
  g_return_if_fail (GTK_IS_MESSAGE (message));
  message->nr_of_buttons = size;
  g_print ("\nmessage->nr_of_buttons : %d", message->nr_of_buttons);
}

GtkWidget *
gtk_message_add_button (GtkMessage * message, gint index, gchar * button_label)
{
  GtkWidget *button;
  g_return_val_if_fail (message != NULL, NULL);
  g_return_val_if_fail (GTK_IS_MESSAGE (message), NULL);
  g_return_val_if_fail (index < message->nr_of_buttons && index >= 0, NULL);
  button = gtk_button_new_with_label (button_label);

  gtk_box_pack_start (GTK_BOX (message->action_area),
		      button, FALSE, FALSE, 0);
  gtk_widget_show (button);
  return button;
}
